using System;
using System.Collections.Generic;
using System.Data;
using System.Text;

using Vevo.Data.Implementation;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Orders;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Domain.Base;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CartRepositoryNoCache : ICartRepository
    {
        #region Private

        private CartItemAccess _cartItemAccess = new CartItemAccess();


        private const string AllCartFields =
            " CartID, CreateDate, UpdateDate ";


        private Cart CreateOneFromRow( DataRow row )
        {
            Cart cart = new Cart();
            cart.CartID = row["CartID"].ToString();
            cart.CreateDate = ConvertUtilities.ToDateTime( row["CreateDate"] );
            cart.UpdateDate = ConvertUtilities.ToDateTime( row["UpdateDate"] );

            return cart;
        }

        #endregion


        #region ICartRepository Members

        public Cart CreateWhole( Cart cart )
        {
            cart.CreateDate = DateTime.Now;
            cart.UpdateDate = cart.CreateDate;

            DataAccess.ExecuteNonQuery(
                "INSERT INTO Cart (CartID, CreateDate, UpdateDate) " +
                "VALUES (@CartID, @CreateDate, @UpdateDate); ",
                DataAccess.CreateParameterString( cart.CartID ),
                DataAccess.CreateParameterDateTime( cart.CreateDate ),
                DataAccess.CreateParameterDateTime( cart.UpdateDate ) );

            _cartItemAccess.SaveAll( cart.CartID, (IEntityList<ICartItem>) cart.CartItems );

            return cart;
        }

        public Cart UpdateWhole( Cart cart )
        {
            cart.UpdateDate = DateTime.Now;

            DataAccess.ExecuteNonQuery(
                "UPDATE Cart " +
                "SET CreateDate = @CreateDate, UpdateDate = @UpdateDate " +
                "WHERE CartID = @CartID; ",
                DataAccess.CreateParameterDateTime( cart.CreateDate ),
                DataAccess.CreateParameterDateTime( cart.UpdateDate ),
                DataAccess.CreateParameterString( cart.CartID ) );

            _cartItemAccess.SaveAll( cart.CartID, (IEntityList<ICartItem>) cart.CartItems );

            return cart;
        }

        public void Delete( string cartID )
        {
            DataAccessContext.CartItemRepository.DeleteByCartID( cartID );

            DataAccess.ExecuteNonQuery(
                "DELETE FROM Cart WHERE CartID = @CartID",
                DataAccess.CreateParameterString( cartID ) );
        }

        public Cart GetOne( string shoppingCartID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllCartFields +
                "FROM Cart " +
                "WHERE CartID = @CartID; ",
                DataAccess.CreateParameterString( shoppingCartID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return Cart.Null;
        }

        #endregion
    }
}
