﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Domain.DataInterfaces;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;
using System.Web.UI;

namespace Vevo.Data
{
    public class CustomerRewardPointRepository : ICustomerRewardPointRepository
    {
        #region Private

        private const string RepositoryName = "CustomerRewardPointRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetOneByOrderIDSignature = "GetOneByOrderID";
        private const string GetByCustomerIDAndStoreIDSignature = "GetByCustomerIDAndStoreID";
        private const string SumCustomerIDAndStoreIDSignature = "SumCustomerIDAndStoreID";
        private const string SearchCustomerRewardPointByCustomerIDAndStoreIDSignature = "SearchCustomerRewardPointByCustomerIDAndStoreID";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( "CustomerRewardPoint" );

        private CustomerRewardPointRepositoryNoCache _repository = new CustomerRewardPointRepositoryNoCache();

        #endregion

        #region Constructors

        public CustomerRewardPointRepository()
        {
        }

        public CustomerRewardPointRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public CustomerRewardPoint Save( CustomerRewardPoint customerRewardPoint )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            CustomerRewardPoint newCustomerRewardPoint = _repository.Save( customerRewardPoint );
            _dataCache.ClearRepositoryCache();
            return newCustomerRewardPoint;
        }

        public void Delete( string customerRewardPointID )
        {
            _repository.Delete( customerRewardPointID );
            _dataCache.ClearRepositoryCache();
        }

        public CustomerRewardPoint GetOne( string customerRewardPointID )
        {
            CustomerRewardPoint result = (CustomerRewardPoint) _dataCache.GetValue( GetOneSignature, _cacheDependency, customerRewardPointID );

            if (result == null)
            {
                result = _repository.GetOne( customerRewardPointID );
                _dataCache.SetValue( GetOneSignature, result, customerRewardPointID );
            }

            return result;
        }

        public CustomerRewardPoint GetOneByOrderID( string orderID )
        {
            CustomerRewardPoint result = (CustomerRewardPoint) _dataCache.GetValue( GetOneByOrderIDSignature, _cacheDependency, orderID );

            if (result == null)
            {
                result = _repository.GetOneByOrderID( orderID );
                _dataCache.SetValue( GetOneByOrderIDSignature, result, orderID );
            }

            return result;
        }

        public IList<CustomerRewardPoint> GetByCustomerIDAndStoreID( string customerID, string storeID )
        {
            IList<CustomerRewardPoint> list = (IList<CustomerRewardPoint>) _dataCache.GetValue(
                GetByCustomerIDAndStoreIDSignature,
                _cacheDependency,
                customerID,
                storeID );

            if (list == null)
            {
                list = _repository.GetByCustomerIDAndStoreID( customerID, storeID );
                _dataCache.SetValue(
                    GetByCustomerIDAndStoreIDSignature,
                    customerID,
                    storeID );
            }

            return list;
        }

        public decimal SumCustomerIDAndStoreID( string customerID, string storeID )
        {
            decimal result = ConvertUtilities.ToDecimal(
                _dataCache.GetValue( SumCustomerIDAndStoreIDSignature, _cacheDependency, customerID, storeID ) );

            if (result == 0)
            {
                result = _repository.SumCustomerIDAndStoreID( customerID, storeID );
                _dataCache.SetValue(
                    SumCustomerIDAndStoreIDSignature,
                    result,
                    customerID,
                    storeID );
            }

            return result;
        }

        public IList<CustomerRewardPoint> SearchCustomerRewardPointByCustomerIDAndStoreID(
            string customerID,
            string storeID,
            string sortBy )
        {
            IList<CustomerRewardPoint> list = (IList<CustomerRewardPoint>) _dataCache.GetValue(
                SearchCustomerRewardPointByCustomerIDAndStoreIDSignature,
                _cacheDependency,
                customerID,
                storeID,
                sortBy );

            if (list == null)
            {
                list = _repository.SearchCustomerRewardPointByCustomerIDAndStoreID( customerID, storeID, sortBy );
            }

            return list;
        }

        public IList<CustomerRewardPoint> SearchCustomerRewardPointByCustomerIDAndStoreID(
            string customerID,
            string storeID,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchCustomerRewardPointByCustomerIDAndStoreIDSignature,
                _cacheDependency,
                customerID,
                storeID,
                sortBy,
                startIndex,
                endIndex );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchCustomerRewardPointByCustomerIDAndStoreID(
                    customerID,
                    storeID,
                    sortBy,
                    startIndex,
                    endIndex,
                    out howManyItems );

                result.Second = howManyItems;

                _dataCache.SetValue(
                    SearchCustomerRewardPointByCustomerIDAndStoreIDSignature,
                    result,
                    customerID,
                    storeID,
                    sortBy,
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<CustomerRewardPoint>) result.First;
        }

        #endregion

    }
}
