﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Domain.Discounts;
using Vevo.Shared.DataAccess;
using System.Data;
using Vevo.Shared.Utilities;

namespace Vevo.Data.Implementation
{
    internal class CouponCustomerAccess
    {
        private const string AllCouponCustomerFields =
           " CouponID, UserName, Quantity ";

        private CouponCustomer CreateOneFromRow( DataRow row )
        {
            CouponCustomer couponCustomer = new CouponCustomer();
            couponCustomer.CouponID = row["CouponID"].ToString();
            couponCustomer.CustomerUserName = row["UserName"].ToString();
            couponCustomer.CurrentQuantity = ConvertUtilities.ToInt32( row["Quantity"] );

            return couponCustomer;
        }

        public CouponCustomer Create( CouponCustomer couponCustomer )
        {
            DataAccess.ExecuteNonQuery(
                "INSERT INTO CouponCustomer " +
                " (" + AllCouponCustomerFields + ") " +
                "VALUES (@CouponID, @UserName, @Quantity);",
                DataAccess.CreateParameterString( couponCustomer.CouponID ),
                DataAccess.CreateParameterString( couponCustomer.CustomerUserName ),
                DataAccess.CreateParameterDecimal( couponCustomer.CurrentQuantity )
            );

            return couponCustomer;
        }

        public CouponCustomer Update( CouponCustomer couponCustomer )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE CouponCustomer " +
                "SET Quantity = @Quantity " +
                "WHERE CouponID = @CouponID AND UserName = @UserName ;",
                DataAccess.CreateParameterString( couponCustomer.CouponID ),
                DataAccess.CreateParameterString( couponCustomer.CustomerUserName ),
                DataAccess.CreateParameterDecimal( couponCustomer.CurrentQuantity )
            );

            return couponCustomer;
        }

        public void Delete( string couponID,string userName )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM CouponCustomer " +
                "WHERE CouponID = @CouponID AND UserName = @UserName ;",
                DataAccess.CreateParameterString( couponID ),
                DataAccess.CreateParameterString( userName )
            );
        }

        public void DeleteByCouponID( string couponID)
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM CouponCustomer " +
                "WHERE CouponID = @CouponID ;",
                DataAccess.CreateParameterString( couponID )
            );
        }


        public void DeleteAll()
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM CouponCustomer;"
            );
        }

        public CouponCustomer GetOne( string customerID, string userName )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllCouponCustomerFields +
                "FROM CouponCustomer " +
                "WHERE CouponID = @CouponID AND UserName = @UserName ;",
                DataAccess.CreateParameterString( customerID ),
                DataAccess.CreateParameterString( userName )
                 );
            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return null;
        }

        public IList<CouponCustomer> GetAllByCouponID( string couponID)
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllCouponCustomerFields +
                "FROM CouponCustomer " +
                "WHERE CouponID = @CouponID ;",
                DataAccess.CreateParameterString( couponID )
                 );
            if (table.Rows.Count > 0)
            {
                List<CouponCustomer> list = new List<CouponCustomer>();
                foreach (DataRow row in table.Rows)
                {
                    list.Add(CreateOneFromRow( row ));
                }
                return list;
            }
            else
                return null;
        }
    }
}
