using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.Discounts;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data.Implementation;

namespace Vevo.Data.Implementation
{
    internal class DiscountRuleAccess : BaseListedDataAccess<string, DiscountRule>
    {
        #region Private

        private DiscountRule CreateOneDiscountRuleFromRow( DataRow row )
        {
            DiscountRule discountRule = new DiscountRule();

            discountRule.DiscountRuleID = row["DiscountRuleID"].ToString();
            discountRule.DiscountGroupID = row["DiscountGroupID"].ToString();
            discountRule.ToItems = Convert.ToInt32( row["ToItems"] );
            discountRule.Percentage = ConvertUtilities.ToDouble( (Convert.ToDecimal( row["Percentage"] ).ToString( "f2" )) );
            discountRule.Amount = decimal.Parse( Convert.ToDecimal( row["Amount"] ).ToString( "f2" ) );

            return discountRule;
        }

        private IList<DiscountRule> ConvertDiscountRuleTableToList( DataTable table )
        {
            IList<DiscountRule> list = new List<DiscountRule>();

            foreach (DataRow row in table.Rows)
            {
                list.Add( CreateOneDiscountRuleFromRow( row ) );
            }

            return list;
        }

        #endregion


        #region Protected

        protected override void Create( string discountGroupID, DiscountRule discountRule )
        {
            discountRule.DiscountRuleID = DataAccess.ExecuteScalar(
                "INSERT INTO DiscountRule( DiscountGroupID, ToItems, Percentage, Amount ) " +
                "VALUES ( @DiscountGroupID, @ToItems, @Percentage, @Amount );" +
                "SELECT @@Identity;",
                DataAccess.CreateParameterString( discountGroupID ),
                DataAccess.CreateParameterInt32( discountRule.ToItems ),
                DataAccess.CreateParameterDouble( discountRule.Percentage ),
                DataAccess.CreateParameterDecimalMoney( discountRule.Amount ) );

            discountRule.DiscountGroupID = discountGroupID;
        }

        protected override void Update( string discountGroupID, DiscountRule discountRule )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE DiscountRule " +
                "SET DiscountGroupID = @DiscountGroupID, " +
                "    ToItems = @ToItems, " +
                "    Percentage = @Percentage, " +
                "    Amount = @Amount " +
                "WHERE DiscountRuleID = @DiscountRuleID;",
                DataAccess.CreateParameterString( discountRule.DiscountGroupID ),
                DataAccess.CreateParameterInt32( discountRule.ToItems ),
                DataAccess.CreateParameterDouble( discountRule.Percentage ),
                DataAccess.CreateParameterDecimalMoney( discountRule.Amount ),
                DataAccess.CreateParameterString( discountRule.DiscountRuleID ) );
        }

        protected override void Delete( string discountGroupID, DiscountRule discountRule )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM DiscountRule " +
                "WHERE DiscountRuleID = @DiscountRuleID;",
                DataAccess.CreateParameterString( discountRule.DiscountRuleID ) );
        }

        #endregion


        #region Public Methods

        public IList<DiscountRule> GetByDiscountGroupID( string discountGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT * FROM DiscountRule " +
                "WHERE DiscountRule.DiscountGroupID = @DiscountGroupID " +
                "ORDER BY ToItems; ",
                DataAccess.CreateParameterString( discountGroupID ) );

            return ConvertDiscountRuleTableToList( table );
        }

        #endregion

    }
}
