using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;

namespace Vevo.Data
{
    public class SearchFilterHelperNew
    {
        #region Private Method

        private string CreateCommandNoPrefix(
            SearchFilter searchFilter,
            string sqlField,
            ArrayList dbParametersToAppend
            )
        {
            string command = String.Empty;
            bool boolValue;
            bool requireAnd = false;

            if (String.IsNullOrEmpty( sqlField ))
                sqlField = searchFilter.FieldName;

            switch (searchFilter.FilterType)
            {
                case SearchFilter.SearchFilterType.Text:
                    if (!String.IsNullOrEmpty( searchFilter.Value1.Trim() ))
                    {
                        command = String.Format( " {0} LIKE @{1} ", sqlField, searchFilter.FieldName );
                        dbParametersToAppend.Add(
                            DataAccess.CreateParameterString( "%" + ReplaceSpecialCharecter( searchFilter.Value1.Trim() ) + "%" ) );
                    }
                    break;

                case SearchFilter.SearchFilterType.Boolean:
                    if (Boolean.TryParse( searchFilter.Value1, out boolValue ))
                    {
                        command = String.Format( " {0} = @{1} ", sqlField, searchFilter.FieldName );
                        dbParametersToAppend.Add( DataAccess.CreateParameterBool( boolValue ) );
                    }
                    break;

                case SearchFilter.SearchFilterType.ValueRange:
                    if (!String.IsNullOrEmpty( searchFilter.Value1.Trim() ))
                    {
                        command += String.Format( " {0} >= @{1} ", sqlField, searchFilter.FieldName );
                        dbParametersToAppend.Add(
                            DataAccess.CreateParameterDouble( Convert.ToDouble( searchFilter.Value1.Trim() ) ) );
                        requireAnd = true;
                    }

                    if (!String.IsNullOrEmpty( searchFilter.Value2.Trim() ))
                    {
                        if (requireAnd)
                            command += "AND ";

                        command += String.Format( " {0} <= @{1} ", sqlField, searchFilter.FieldName + "1" );
                        dbParametersToAppend.Add(
                            DataAccess.CreateParameterDouble( Convert.ToDouble( searchFilter.Value2.Trim() ) ) );
                    }
                    break;

                case SearchFilter.SearchFilterType.Date:
                    if (!String.IsNullOrEmpty( searchFilter.Value1.Trim() ))
                    {
                        command += String.Format( " {0} >= @{1} ", sqlField, searchFilter.FieldName );

                        DateTime dateOrig = Convert.ToDateTime( searchFilter.Value1.Trim(), Culture.USCultureInfo );
                        dbParametersToAppend.Add(
                            DataAccess.CreateParameterDateTime( DateTimeUtilities.StartTimeOfDay( dateOrig ) ) );

                        requireAnd = true;
                    }

                    if (!String.IsNullOrEmpty( searchFilter.Value2.Trim() ))
                    {
                        if (requireAnd)
                            command += "AND ";

                        command += String.Format( " {0} <= @{1} ", sqlField, searchFilter.FieldName + "1" );

                        DateTime dateOrig = Convert.ToDateTime( searchFilter.Value2.Trim(), Culture.USCultureInfo );
                        dbParametersToAppend.Add(
                            DataAccess.CreateParameterDateTime( DateTimeUtilities.EndTimeOfDay( dateOrig ) ) );
                    }
                    break;
            }

            return command;
        }

        private string CreateExtraFilter( string extraFilter )
        {
            if (!String.IsNullOrEmpty( extraFilter ))
                return " AND " + extraFilter + " ";
            else
                return String.Empty;
        }

        private string ReplaceSpecialCharecter( string str )
        {
            string[] allString = str.Split( '[' );
            for (int i = 0; i < allString.Length; i++)
            {
                allString[i] = allString[i].Replace( "]", "[]]" );
            }
            return string.Join( "[[]", allString );
        }

        private string RevertField( string fieldName )
        {
            string[] fieldNames = fieldName.Split( '.' );
            return fieldNames[fieldNames.Length - 1];
        }

        #endregion


        #region Public Methods

        public string CreateCommandWithAnd(
            SearchFilter searchFilter,
            string sqlField,
            ArrayList dbParametersToAppend
            )
        {
            string command = CreateCommandNoPrefix( searchFilter, sqlField, dbParametersToAppend );
            if (!String.IsNullOrEmpty( command ))
                return " AND " + command;
            else
                return String.Empty;
        }

        public string CreateCommandWithWhere(
            SearchFilter SearchFilter,
            string sqlField,
            ArrayList dbParametersToAppend
            )
        {
            return CreateCommandWithWhere( SearchFilter, sqlField, String.Empty, dbParametersToAppend );
        }

        public string CreateCommandWithWhere(
            SearchFilter searchFilter,
            string sqlField,
            string extrafilter,
            ArrayList dbParametersToAppend
            )
        {
            string command = CreateCommandNoPrefix( searchFilter, sqlField, dbParametersToAppend );
            if (!String.IsNullOrEmpty( command ))
                return " WHERE " + command + CreateExtraFilter( extrafilter );
            else
                return " WHERE 1 = 1 " + CreateExtraFilter( extrafilter );
        }

        #endregion
    }
}
