﻿using System;
using System.Collections.Generic;

using Vevo.Domain.Base;
using Vevo.Domain.Products;
using System.Data;
using Vevo.Base.Domain;

namespace Vevo.Domain.DataInterfaces
{
    public interface ICategoryHelperRepository
    {
        void Delete( string categoryID );
        void DeleteLocalesByCultureID( string cultureID );
        void UpdateSortOrder( string[] sortedCategoryIDs );

        DataTable GetOne( Culture culture, string categoryID );
        DataTable GetOneByUrlName( Culture culture, string urlName );
        DataTable GetAll( Culture culture, string sortBy );
        DataTable GetAll( Culture culture, string sortBy, BoolFilter isEnabled );

        DataTable GetAllWithoutRootCategory( Culture culture, string sortBy );

        DataTable GetByParentID(
            Culture culture,
            string parentID,
            string sortBy,
            BoolFilter isEnabled );

        DataTable GetByParentIDAndRootID(
            Culture culture,
            string parentCategoryID,
            string rootID,
            string sortBy,
            BoolFilter isEnabled );

        DataTable GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled );

        DataTable GetRootCategory(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled );

        DataTable GetByParentUrlName(
             Culture culture,
             string parentName,
             string sortBy,
             BoolFilter isEnabled,
             int startIndex,
             int endIndex );

        DataTable GetLocales( string ID );
        IList<TableSchemaItem> GetTableSchema();

        bool IsLeaf( string parentID );
        bool IsNotLeaf( string parentID );
        bool IsUrlNameNotLeaf( string parentCategoryUrlName );
        void RefreshUrlName( string categoryID, string name );
        bool HasItInDiscountGroup( string discountgroupID );

        DataTable SearchByRootID(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            string rootID
            );

        DataTable SearchRoot(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
            );

        // ---- For ProductRepository ----
        DataTable GetByCategoryDepartmentID(
            Culture culture,
            string categorydepartmentID,
            string sortBy,
            BoolFilter isEnabled,
            bool isIncludeProductKit,
            string storeID,
            int startIndex,
            int endIndex
        );

        DataTable GetByCategoryDepartmentID(
            Culture culture,
            string categorydepartmentID,
            string sortBy,
            BoolFilter isEnabled,
            string storeID,
            int startIndex,
            int endIndex
        );

        DataTable GetByCategoryDepartmentID(
            Culture culture,
            string categorydepartmentID,
            string sortBy,
            BoolFilter isEnabled,
            string storeID
            );

        DataTable GetAllByBestSelling(
            Culture culture,
            int numberOfResult,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootID,
            string enabledListCondition,
            bool isIncludeProductKit
            );

        DataTable GetAllByRandom(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootID,
            string enabledListCondition,
            bool isIncludeProductKit
        );

        void UpdateSortOrder( string categoryID, string[] sortedProductIDs );
    }
}
