﻿using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain.Base;
using System.Collections;

namespace Vevo.Domain.Products
{
    public class ProductKitGroup : ILocaleLoader, ICloneable
    {
        #region Enum Type
        public enum ProductKitGroupType
        {
            Radio,
            DropDown,
            Checkbox,
            Unknown
        }
        #endregion


        #region Private

        private string _productKitGroupID = "0";
        private ProductKitGroupType _type = ProductKitGroupType.Unknown;
        private bool _isRequired = false;

        private Culture _defaultCulture;
        private LocaleCollection<ProductKitGroupLocale> _locales;

        private IList<ProductKitItem> _productKitItems;

        #endregion


        #region Public Properties

        public static ProductKitGroup Null
        {
            get { return NullProductKitGroup.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string ProductKitGroupID { get { return _productKitGroupID; } set { _productKitGroupID = value; } }
        public ProductKitGroupType Type { get { return _type; } set { _type = value; } }
        public bool IsRequired { get { return _isRequired; } set { _isRequired = value; } }

        public Culture DefaultCulture { get { return _defaultCulture; } set { _defaultCulture = value; } }
        public LocaleCollection<ProductKitGroupLocale> Locales { get { return _locales; } }

        //------------------------------------------------------------
        // Locale items
        public string Name
        {
            get { return Locales[DefaultCulture].Name; }
            set { Locales.GetNoLoad( DefaultCulture ).Name = value; }
        }
        public string Description
        {
            get { return Locales[DefaultCulture].Description; }
            set { Locales.GetNoLoad( DefaultCulture ).Description = value; }
        }
        // End Locale items
        //------------------------------------------------------------

        #endregion


        #region Custom Properties

        public IList<ProductKitItem> ProductKitItems
        {
            get
            {
                if (_productKitItems == null)
                    _productKitItems =
                        DataAccessContext.ProductKitGroupRepository.GetProductKitItems( ProductKitGroupID, DefaultCulture );

                return _productKitItems;
            }
            set
            {
                _productKitItems = value;
            }
        }

        #endregion


        #region Public Constructors

        public ProductKitGroup( Culture defaultCulture )
        {
            _defaultCulture = defaultCulture;
            _locales = new LocaleCollection<ProductKitGroupLocale>( this );
        }

        #endregion


        #region ILocaleLoader Members

        public IList<ILocale> GetLocales()
        {
            List<ILocale> result = new List<ILocale>();

            foreach (ProductKitGroupLocale locale in DataAccessContext.ProductKitGroupRepository.GetLocales( ProductKitGroupID ))
                result.Add( locale );

            return result;
        }

        #endregion


        #region IClonable Members

        // Copy the whole aggregate (including locale)
        public object Clone()
        {
            ProductKitGroup newObject = (ProductKitGroup) MemberwiseClone();
            newObject._locales = (LocaleCollection<ProductKitGroupLocale>) Locales.Clone();

            return newObject;
        }

        #endregion


        #region Public Methods

        #endregion
    }
}
