using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Domain.Orders;
using Vevo.Domain.Users;
using Vevo.Domain.Exceptions;

namespace Vevo.Domain.Shipping.Custom
{
    public class ByOrderTotalShippingMethod : CustomShippingMethod
    {
        #region Private

        private decimal GetByOrderTotalShippingCost(
            CartItemGroup cartItemGroup,
            WholesaleStatus wholesaleStatus,
            decimal discount )
        {

            CartItemGroup nonOverridden = new CartItemGroup( GetNonOverriddenShippingCartItems( cartItemGroup ) );
            
            bool option = DataAccessContext.Configurations.GetBoolValue( "ShippingIncludeDiscount" );
            decimal orderTotalWithDiscount = 0.0m;

            if (option)
            {
                orderTotalWithDiscount = nonOverridden.GetSubtotal( wholesaleStatus ) - discount;
            }
            else
            {
                orderTotalWithDiscount = nonOverridden.GetSubtotal( wholesaleStatus );
            }

            ShippingOrderTotalRate orderTotalRate = SelectApplicableOrderTotalRate( orderTotalWithDiscount );
            return orderTotalRate.OrderTotalRate;
        }

        #endregion

        #region Constructors

        public ByOrderTotalShippingMethod(
            ShippingOption shippingOption,
            ShippingChoice selectedShippingChoice )     // ignored in non-real-time shipping option

            : base( shippingOption )
        {
        }

        #endregion

        #region Public Constances

        public const string ShippingOptionTypeName = "ByOrderTotal";

        #endregion

        #region Public Methods

        public ShippingOrderTotalRate SelectApplicableOrderTotalRate( decimal orderTotal )
        {
            IList<ShippingOrderTotalRate> orderTotalRates =
                DataAccessContext.ShippingOrderTotalRateRepository.GetAllByShippingID(
                    ShippingOption.ShippingID,
                    "ToOrderTotal" );

            for (int i = 0; i < orderTotalRates.Count; i++)
            {
                if (orderTotalRates[i].ToOrderTotal >= orderTotal)
                    return orderTotalRates[i];
            }

            throw new ShippingException(
                "Order Total is out of range. "
                + "Order Total: " + orderTotal + ", Maximum Order Total: "
                + orderTotalRates[orderTotalRates.Count - 1].OrderTotalRate
                + ", ShippingID: " + ShippingOption.ShippingID );
        }

        public override decimal GetShippingCost(
            CartItemGroup cartItemGroup,
            WholesaleStatus wholesaleStatus,
            decimal discount )
        {
            if (IsFreeShippingForNonOverriddenItems( cartItemGroup, wholesaleStatus ))
                return GetOverriddenShippingAmount( cartItemGroup );
            else
                return GetOverriddenShippingAmount( cartItemGroup )
                    + GetByOrderTotalShippingCost( cartItemGroup, wholesaleStatus, discount );
        }

        #endregion
    }
}
