using System;
using System.Collections;
using System.Data;
using System.IO;
using System.Text;
using Vevo.Shared.Utilities;

namespace Vevo.Reports.Exporters
{
    public class CustomerReportExporter
    {
        private DataTable _table;
        public CustomerReportExporter()
        {
          
        }

        private string GenerateHeader( CustomerReportType customerReportType )
        {
            if (customerReportType == CustomerReportType.UserRegistration)
            {
                return "\"Date/Time\",\"Register Customer\"";
            }
            else
            {
                return "\"UserName\",\"Register Date\",\"Total\"," +
                    "\"Number Of Order\",\"Average\",\"WholeSale Level\"";
            }
        }

        private string GenerateDetail( DataRow row, CustomerReportType customerReportType )
        {
            if (customerReportType == CustomerReportType.UserRegistration)
            {
                string period, registerCustomer;
                period = row["RegisterPeriod"].ToString();
                registerCustomer = row["RegisterCustomer"].ToString();

                return period + "," + registerCustomer;
            }
            else
            {
                string userName, registerDate, total, numberOfOrder, average, wholeSaleLevel;
                userName = row["UserName"].ToString();
                registerDate = row["RegisterDate"].ToString();
                total = row["Total"].ToString();
                numberOfOrder = row["NumberOfOrder"].ToString();
                average = row["Average"].ToString();
                wholeSaleLevel = row["WholeSaleLevel"].ToString();
                return userName + "," + registerDate + "," + total + "," + numberOfOrder + ","
                    + average + "," + wholeSaleLevel;
            }
        }

        private string GenerateFooter()
        {
            int TotalRegister = 0;
            foreach (DataRow row in _table.Rows)
            {
                TotalRegister += ConvertUtilities.ToInt32( row["RegisterCustomer"].ToString() );
            }
            return "\"Total\"" + "," + TotalRegister;
        }
        private string CreateExportFile(
            DataTable table,
            string filePhysicalPath,
            string filePathName,
            CustomerReportType customerReportType,
            out string message,
            out string message1,
            out string fileNameLinkText )
        {
            string filePhysicalPathName = filePhysicalPath + filePathName;
            _table = table;
            if (table.Rows.Count > 0)
            {
                StreamWriter writer = new StreamWriter( filePhysicalPathName, false );
                writer.WriteLine( GenerateHeader( customerReportType ) );

                string details;
                foreach (DataRow row in table.Rows)
                {
                    details = GenerateDetail( row, customerReportType );
                    writer.WriteLine( details );
                }

                if (customerReportType == CustomerReportType.UserRegistration)
                    writer.WriteLine( GenerateFooter() );
                
                writer.Dispose();
            }
            if (table.Rows.Count == 0)
            {
                message = "No data to export.";
                message1 = "";
                fileNameLinkText = "";
                return "";
            }
            else
            {
                message = "Export file is successfully create.";
                message1 = "Download ";
                fileNameLinkText = Path.GetFileName( filePathName );
                return "../DownloadFile.aspx?FilePath=" + "../" + filePathName;
            }
        }

        public string ExportCustomerReportData(
            CustomerReportType customerReportType,
            PeriodType period,
            string filePhysicalPath,
            DateTime startDate,
            DateTime endDate,
            out string message,
            out string message1,
            out string fileNameLinkText )
        {
            CustomerReportBuilder customerReportBuilder = new CustomerReportBuilder();
            DataTable table = customerReportBuilder.GetCustomerReportData(
                    period,
                    startDate,
                    endDate );

            string filePathName = "Export/" +
            period + " Report_from_" +
            startDate.Month + "_" + startDate.Day + "_" + startDate.Year + "_to_" +
            endDate.Month + "_" + endDate.Day + "_" + endDate.Year + ".csv";

            string fileNameLinkUrl = CreateExportFile(
                table,
                filePhysicalPath,
                filePathName,
                customerReportType,
                out message,
                out message1,
                out fileNameLinkText );

            return fileNameLinkUrl;
        }

        public string ExportCustomerReportData(
            CustomerReportType customerReportType,
            PeriodType period,
            string numDisplay,
            string filePhysicalPath,
            DateTime startDate,
            DateTime endDate,
            out string message,
            out string message1,
            out string fileNameLinkText )
        {
            string sort = String.Empty;
            if (customerReportType == CustomerReportType.TopNumberOfProductBuyer)
            {
                sort = "NumberOfOrder DESC";
            }
            else if (customerReportType == CustomerReportType.TopProductTotalPriceBuyer)
            {
                sort = "Total DESC";
            }
            CustomerReportBuilder customerReportBuilder = new CustomerReportBuilder();
            DataTable table = customerReportBuilder.GetTopCustomerReportData(
                sort,
                period,
                customerReportType,
                numDisplay,
                startDate,
                endDate );

            string filePathName = "Export/Customer_Top" +
            numDisplay + "_" +
            customerReportType.ToString().Substring( 4 ) + " Report_from_" +
            startDate.Month + "_" + startDate.Day + "_" + startDate.Year + "_to_" +
            endDate.Month + "_" + endDate.Day + "_" + endDate.Year + ".csv";

            string fileNameLinkUrl = CreateExportFile(
                table,
                filePhysicalPath,
                filePathName,
                customerReportType,
                out message,
                out message1,
                out fileNameLinkText );

            return fileNameLinkUrl;
        }
    }
}
