using System;
using System.Data;
using System.Collections.Generic;
using System.Configuration;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Vevo.DataAccessLib.Cart;
using Vevo.Domain;
using Vevo.Domain.Orders;
using Vevo.Domain.Products;
using Vevo.WebAppLib;
using Vevo.WebUI;
using Vevo.WebUI.Users;
using Vevo.Domain.Stores;

namespace Vevo
{
    /// <summary>
    /// Summary description for GoogleAnalyticUtilities
    /// </summary>
    public static class GoogleAnalyticUtilities
    {
        private static string FormatNumber( decimal num )
        {
            if (num == 0)
                return "0";
            else
                return string.Format( "{0:n2}", num ).ToString();
        }

        private static string GetCategoryName( string productID )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( StoreContext.Culture, productID,Store.Null.StoreID );
            string categoryID = product.CategoryIDs[0];
            Category category = DataAccessContext.CategoryRepository.GetOne( StoreContext.Culture, categoryID );
            return category.CreateFullCategoryPath();
        }

        public static string RegisterTrackingScript()
        {
            Store store = StoreContext.CurrentStore;
            string script = string.Empty;
            if (DataAccessContext.Configurations.GetBoolValue( "GoogleAnalyticsEnabled", store ) && !HttpContext.Current.Request.Url.IsLoopback)
            {
                if (!DataAccessContext.Configurations.GetBoolValue( "GoogleAnalyticsCustomCodeEnabled", store ))
                {
                    string account = DataAccessContext.Configurations.GetValueNoThrow( "GoogleAnalyticsAccount",store );
                    if (!string.IsNullOrEmpty( account ))
                    {
                        script = "<script type=\"text/javascript\">"
                            + "var _gaq = _gaq || [];"
                            + "_gaq.push(['_setAccount', '" + account + "']);"
                            + "_gaq.push(['_trackPageview']);"
                            + "(function() {"
                            + "var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;"
                            + "ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';"
                            + "var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);"
                            + "})();"
                            + "</script>";
                    }
                }
                else
                {
                    string customcode = DataAccessContext.Configurations.GetValueNoThrow( "GoogleAnalyticsCustomCode",store );
                    if (!string.IsNullOrEmpty( customcode ))
                    {
                        script = customcode;
                    }
                }
            }
            return script;
        }

        private static string RepalceBRTag( string str )
        {
            Regex reg = new Regex( "<br>", RegexOptions.IgnoreCase );
            return reg.Replace( str, "\\n" );
        }

        public static string RegisterTransactionScript( string orderID )
        {
            string account = DataAccessContext.Configurations.GetValue( "GoogleAnalyticsAccount" );
            string script = string.Empty;
            if ((!string.IsNullOrEmpty( account ))
                && DataAccessContext.Configurations.GetBoolValue( "GoogleAnalyticsEnabled" )
                && (!HttpContext.Current.Request.Url.IsLoopback)
                && orderID != "0")
            {
                Order order = DataAccessContext.OrderRepository.GetOne( orderID );
                IList<OrderItem> orderItemList = DataAccessContext.OrderItemRepository.GetByOrderID( orderID );

                string scriptItems = string.Empty;
                for (int i = 0; i < orderItemList.Count; i++)
                {
                    OrderItem item = orderItemList[i];
                    scriptItems += "pageTracker._addItem(\"" + orderID + "\", " +    // Order ID : "1234"
                        "\"" + item.Sku + "\", " +                             // SKU : "DD44"
                        "\"" + RepalceBRTag( item.Name ) + "\", " +                            // Product Name : "T-Shirt"
                        "\"" + GetCategoryName( item.ProductID ) + "\", " +    // Category : "Green Medium"
                        "\"" + item.UnitPrice.ToString() + "\", " +                       // Price : "11.99"
                        "\"" + item.Quantity.ToString() + "\" " +                        // Quantity : "1"
                        ");";
                }

                script = "<script type=\"text/javascript\">" +
                    "var gaJsHost = ((\"https:\" == document.location.protocol) ? \"https://ssl.\" : \"http://www.\");" +
                    "document.write(unescape(\"%3Cscript src='\" + gaJsHost + \"google-analytics.com/ga.js'" +
                    "type='text/javascript'%3E%3C/script%3E\"));" +
                    "</script>" +
                    "<script type=\"text/javascript\">" +
                    "var pageTracker = _gat._getTracker(\"" + account + "\");" +
                    "pageTracker._initData();" +
                    "pageTracker._trackPageview();" +
                    "pageTracker._addTrans( \"" + orderID + "\", " +                     // Order ID : "1234"
                    "\"" + order.Billing.FirstName + " " + order.Billing.LastName + "\", " +               // Affiliation : "Mountain View"
                    "\"" + FormatNumber( order.Total ) + "\", " +                                             // Total : "11.99"
                    "\"" + FormatNumber( order.Tax ) + "\", " +                                               // Tax : "1.29"
                    "\"" + FormatNumber( order.ShippingCost ) + "\", " +                                     // Shipping :"5"
                    "\"" + order.Shipping.City + "\", " +                                     // City : "San Jose"
                    "\"" + order.Shipping.State + "\", " +                                    // State : "California"
                    "\"" + AddressUtilities.GetOnlyCountryNameByCountryCode( order.Shipping.Country ) + "\");";        // Country : "USA"
                script += scriptItems;
                script += "pageTracker._trackTrans();" +
                    "</script>";
            }
            return script;
        }
    }
}
